#include "semaphore.hpp"

namespace osal {

/**
 * @brief Constructeur generique pour un Sempahore FreeRTOS
 *
 * @param name				Nom du semaphore (pour l'identification)
 * @param semaphoreHandle 	Handle permettant de prendre et de donner le
 * semaphore
 */
FreeRtosSemaphore::FreeRtosSemaphore(const char* name,
                                     SemaphoreHandle_t semaphoreHandle)
    : m_name(name) {
	handle = semaphoreHandle;
	if (handle == nullptr) {
		for (;;) { /* simulate exception thrown "insufficient heap space" */
		}
	}
}

/**
 * @brief bloquer une ressource du sémaphore
 *
 * @param timeMS			Temps d'attente maximum du sempahore (si timeMS ==
 * 							portMAX_DELAY, pas de limite de temps)
 *
 */
bool FreeRtosSemaphore::take(unsigned long timeMS = MAX_DELAY_TICK) {
	return static_cast<bool>(xSemaphoreTake(handle, timeMS));
}

/**
 * @brief libérer une ressource du sémaphore
 *
 */
bool FreeRtosSemaphore::give() {
	return static_cast<bool>(xSemaphoreGive(handle));
}

}  // namespace osal
