/**
 * FreeRtos Includes
 */
#include <FreeRTOS.h>
#include "queue.h"
#include "semphr.h"

namespace {
constexpr unsigned long MAX_DELAY_TICK = portMAX_DELAY;
}

namespace osal {

/**
 * @brief Interface générique d'un semaphore
 *
 */
class ISemaphore {
   public:
	virtual bool take() = 0;
	virtual bool give() = 0;

	virtual ~ISemaphore() {}
};

/**
 * @brief Classe générique d'un semaphore dans FreeRTOS
 *
 */
class FreeRtosSemaphore : public ISemaphore {
   public:
	FreeRtosSemaphore(const char* name, SemaphoreHandle_t handle);

	virtual bool take(unsigned long timeMS);

	virtual bool take() { return take(MAX_DELAY_TICK); }

	virtual bool give();

	virtual ~FreeRtosSemaphore() {}

   protected:
	virtual SemaphoreHandle_t initSemaphore() = 0;

	StaticSemaphore_t buffer;
	SemaphoreHandle_t handle;
	const char* m_name;
};

}  // namespace osal
