#include <stdio.h>
#include <array>
#include <string>

#include "FreeRTOS.h"
#include "queue.h"  /* RTOS queue related API prototypes. */
#include "semphr.h" /* Semaphore related API prototypes. */
#include "task.h"   /* RTOS task related API prototypes. */
#include "timers.h" /* Software timer related API prototypes. */

#include <error-handler.hpp>
#include <rtos.hpp>
#include <task.hpp>

#include <stm32f0xx.h>
#include <gpio.hpp>

using namespace osal;
using namespace stm32f072xb;
// https://www.st.com/resource/en/reference_manual/dm00031936-stm32f0x1stm32f0x2stm32f0x8-advanced-armbased-32bit-mcus-stmicroelectronics.pdf

constexpr uint32_t ledPin = 5;

void enableGpio() {
	// Gpio gpio = Gpio();
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN);  // enable GPIOA (p.120)
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOCEN);  // enable GPIOC (p.120)
}

void setupLed(const uint32_t ledPin) {
	// LED
	CLEAR_BIT(GPIOA->MODER, (0x3 << ledPin * 2));
	GPIOA->MODER |= (0x1 << (ledPin * 2));    // Output mode (p.157)
	CLEAR_BIT(GPIOA->OTYPER, (1 << ledPin));  // Push-pull output (p.157)
}

#define mainQUEUE_LENGTH 1
StaticSemaphore_t sem;
StaticSemaphore_t sem2;

class LedOnTask : public Task<> {
   public:
	LedOnTask() : Task("Led On") {}

	bool active = true;

   protected:
	virtual void run() override {
		for (;;) {
			active = !active;
			vTaskDelay(2000 / portTICK_PERIOD_MS);
			if (active) {
				SET_BIT(GPIOA->ODR, 1 << ledPin);  // turn on LED, p.159
			}
		}
	}
};

class LedOffTask : public Task<512> {
   public:
	LedOffTask() : Task("Led Off") {}

	bool active = false;

   protected:
	virtual void run() override {
		for (;;) {
			active = !active;
			vTaskDelay(2000 / portTICK_PERIOD_MS);
			if (active) {
				CLEAR_BIT(GPIOA->ODR, 1 << ledPin);  // turn off LED, p.159
			}
		}
	}
};

int main(void) {
	enableGpio();
	setupLed(ledPin);

	LedOnTask task1;
	task1.start();

	LedOffTask task2;
	task2.start();

	Rtos::startScheduler();

	while (1)
		;

	return 0;
}
